/**
 * @file
 * Triggers the preview button when values in the custom block form are changed.
 */

(function ($, Drupal) {

  // Holds the un-monkey-patched version of Drupal.behaviors.dialog.
  let drupalBehaviorsDialogAttach = null;

  // Used to as a "kill switch" to prevent the dialog from stealing focus on preview reload.
  let preventDialogStealingFocus = false;

  // Whether we have attached event listeners to the CKEDITOR object yet.
  let ckeditor4Bound = false;

  const form_selector = 'form#layout-builder-update-block';

  // Submits the form.
  function submitForm() {
    // Skip preview when CKEditor 4 is maximized. The preview isn't visible
    // anyway, and it sometimes causes weird styling issues.
    if (document.getElementsByClassName('cke_maximized').length) {
      return;
    }

    const $preview = $('.layout-builder-instant-preview');

    // Disable the preview button.
    $preview.prop('disabled', true);

    // Prevent dialog.ajax.js from stealing focus.
    preventDialogStealingFocus = true;

    // Note: .click does not work here.
    // See https://drupal.stackexchange.com/questions/11638/how-to-programmatically-trigger-a-click-on-an-ajax-enabled-form-submit-button
    $preview.mousedown();
  };

  // Cancels the form.
  function triggerCancel(form_selector, e) {
    var cancel = $(form_selector).find('[data-drupal-selector="edit-actions-cancel"]');
    if (cancel.length) {
      cancel.trigger('mousedown');
      e.preventDefault();
      return false;
    }
  };

  // Core's formUpdated event is already debounced by 300ms. Add a small
  // additional debounce, since this event tends to be fired multiple times
  // very quickly after using the Media Library widget.
  debounceSubmitForm = Drupal.debounce(submitForm, 200);
  // Core does not apply the 300ms formUpdated debounce correctly for
  // CKEditor 5, so make ours slower to match the rest of the form elements.
  debounceSubmitFormCKEditor5 = Drupal.debounce(submitForm, 500);

  Drupal.behaviors.layoutBuilderInstantPreview = {

    attach: function (context) {
      // Listen for core's formUpdated event to tell us when a form element has
      // changed.
      once('layoutBuilderInstantPreview', form_selector).forEach((form) => {
        $(form).on('formUpdated', function (e) {
          // Special handling for CKEditor 5, see above.
          if (e.target.getAttribute('data-ckeditor5-id') || e.target.classList.contains('ck-content')) {
            debounceSubmitFormCKEditor5();
          }
          // Do not submit if one of the submit buttons triggered the event to
          // avoid an infinite loop.
          // Do not submit if the form itself triggered the event. After the
          // Media Library widget is used, this event is fired frequently on the
          // form itself for some reason, when any form field is changed.
          // Ignore the Media Library widget. It is handled separately below.
          // Do not submit when editing a link field. Validation errors may
          // occur, and there's no visual change when changing a link anyway.
          else if (e.target.name != 'op' && e.target.tagName != 'FORM' && !e.target.closest('.js-media-library-widget') && !e.target.name?.endsWith('[uri]')) {
            debounceSubmitForm();
          }
        });

        // Trigger "Cancel" button when off canvas tray closed using the close
        // button.
        $(form_selector)
          .closest('.ui-dialog-off-canvas')
          .find('.ui-dialog-titlebar-close')
          .off('click')
          .click(function (e) {
            return triggerCancel(form_selector, e);
          });

        // Trigger "Cancel" button when off canvas tray closed using the ESC
        // key.
        $(form_selector)
          .closest('.ui-dialog-off-canvas')
          .off('keydown')
          .keydown(function (e) {
            if (e.keyCode === 27) {
              return triggerCancel(form_selector, e);
            }
          });
      });

      // The formUpdated is not fired reliably for the Media Library widget.
      // Submit the form if the media library widget is reloaded after adding or
      // removing a media item.
      if (context.classList && context.classList.contains('js-media-library-widget')) {
        debounceSubmitForm();
      }

      // When the preview is rendered, misc/dialog/dialog.ajax.js will steal focus from whatever it was on. Here we
      // monkey-patch Drupal.behaviors.dialog.attach to give ourselves a "kill switch" for when the preview is
      // rendering, that is the preventDialogStealFocus variable.
      if (!drupalBehaviorsDialogAttach && Drupal.behaviors.dialog) {
        drupalBehaviorsDialogAttach = Drupal.behaviors.dialog.attach;
        Drupal.behaviors.dialog.attach = function (context, settings) {
          if (preventDialogStealingFocus && $(context).closest('.ui-dialog-off-canvas').length > 0) {
            preventDialogStealingFocus = false;
          }
          else {
            drupalBehaviorsDialogAttach(context, settings);
          }
        };
      }

      // Instant previews are suppressed when CKEditor 4 is maximized. When the
      // user un-maximizes the editor, update the preview.
      if (typeof CKEDITOR !== 'undefined' && !ckeditor4Bound) {
        ckeditor4Bound = true;
        CKEDITOR.on('instanceCreated', function(e) {
          e.editor.on('maximize', function (e) {
            if (e.data == 2) {
              submitForm();
            }
          });
        });
      }
    },

  }

})(jQuery, Drupal);
